@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2023 OpenVPN Inc <sales@openvpn.net>
+ *  Copyright (C) 2002-2024 OpenVPN Inc <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -195,7 +195,7 @@ check_compression_settings_valid(struct compress_options *info, int msglevel)
     }
 #endif
 #ifndef ENABLE_LZO
-    if (info->alg == COMP_ALG_LZO || info->alg == COMP_ALG_LZ4)
+    if (info->alg == COMP_ALG_LZO)
     {
         msg(msglevel, "OpenVPN is compiled without LZO support. Requested "
             "compression cannot be enabled.");
